@@ -11,6 +11,7 @@
 package eu.emi.security.authn.x509.proxy;
 
 import java.math.BigInteger;
+import java.security.cert.CertificateException;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -19,6 +20,7 @@ import java.util.List;
 import java.util.concurrent.TimeUnit;
 
 import org.bouncycastle.asn1.x509.AttributeCertificate;
+import org.bouncycastle.asn1.x509.KeyUsage;
 
 import eu.emi.security.authn.x509.helpers.proxy.ProxyAddressRestrictionData;
 import eu.emi.security.authn.x509.impl.CertificateUtils;
@@ -40,16 +42,23 @@ public abstract class BaseProxyCertificateOptions
 		CertificateUtils.configureSecProvider();
 	}
 
+	/**
+	 * Key usage value which is used when 
+	 */
+	public static final int DEFAULT_KEY_USAGE = KeyUsage.dataEncipherment
+			| KeyUsage.digitalSignature | KeyUsage.keyEncipherment;
+	
 	public static final int DEFAULT_LIFETIME = 12*3600;
 	private final X509Certificate[] parentChain;
 	
 	private int lifetime = DEFAULT_LIFETIME;
 	private Date notBefore;
-	private ProxyType type = ProxyType.RFC3820;
+	private ProxyType type;
 	private boolean limited = false;
 	private BigInteger serialNumber = null;
 	private int proxyPathLimit = -1;
-	
+	private int proxyKeyUsageMask = -1;
+
 	private List<CertificateExtension> extensions;
 	private ProxyPolicy policy = null;
 	private String[] targetRestrictionPermitted;
@@ -63,6 +72,9 @@ public abstract class BaseProxyCertificateOptions
 	
 	/**
 	 * Create a new proxy cert based on the parent cert chain.
+	 * The default type of the proy generation params will be set to the type of the
+	 * parent chain if it is an consistent proxy chain. If it is mixed proxy chain, 
+	 * or EEC certificate chain then by default RFC proxy type is set.
 	 * @param parentCertChain chain of the issuer
 	 */
 	protected BaseProxyCertificateOptions(X509Certificate[] parentCertChain)
@@ -72,6 +84,35 @@ public abstract class BaseProxyCertificateOptions
 		this.parentChain = Arrays.copyOf(parentCertChain, parentCertChain.length);
 		extensions = new ArrayList<CertificateExtension>();
 		notBefore = new Date();
+		
+		if (ProxyUtils.isProxy(parentCertChain))
+		{
+			ProxyChainType pct;
+			try
+			{
+				pct = new ProxyChainInfo(parentCertChain).getProxyType();
+			} catch (CertificateException e)
+			{
+				throw new IllegalArgumentException("Can not parse the parentCertChain argument", e);
+			}
+			if (pct == ProxyChainType.RFC3820)
+				type = ProxyType.RFC3820;
+			else if (pct == ProxyChainType.DRAFT_RFC)
+				type = ProxyType.DRAFT_RFC;
+			else if (pct == ProxyChainType.LEGACY)
+				type = ProxyType.LEGACY;
+			else
+				type = ProxyType.RFC3820;
+		} else
+			type = ProxyType.RFC3820;
+
+// Removed see issue #64. When creating legacy proxies the requirement to have digSig KU is not formally 
+// enforced (err there is no formal definition) so we can't perform a sanity check here.
+		
+//		Integer parentKU = ProxyGeneratorHelper.getChainKeyUsage(parentCertChain);
+//		if (parentKU != null && ((parentKU & KeyUsage.digitalSignature) == 0))
+//			throw new IllegalArgumentException("The parent certificate chain has no digital signature" +
+//					" bit set in its Key Usage. This chain can not be used to create proxies.");
 	}
 
 	/**
@@ -151,6 +192,31 @@ public abstract class BaseProxyCertificateOptions
 		return notBefore;
 	}
 
+	/**
+	 * @return bit mask of KeyUsage flags which was set for the options object or -1 if nothing was set.
+	 */
+	public int getProxyKeyUsageMask()
+	{
+		return proxyKeyUsageMask;
+	}
+
+	/**
+	 * Sets the mask of the KeyUsage for the resulting proxy certificate. Note that the this is a mask,
+	 * i.e. the flags from this mask are ANDed with the effective KeyUsage of the parent chain.
+	 * <p>
+	 * If this method is not called at all (or called with a negative argument), then the default behavior
+	 * is applied, and the proxy gets a copy of the effective KeyUsage of the parent chain. If no certificate
+	 * in the parent chain has KeyUsage set, then the {@link #DEFAULT_KEY_USAGE} is applied.     
+	 * @param proxyKeyUsageMask The mask to set. Use constants from the {@link KeyUsage} class. The mask must always
+	 * have the {@link KeyUsage#digitalSignature} bit set.
+	 * @throws IllegalArgumentException if the argument has no {@link KeyUsage#digitalSignature} bit set
+	 */
+	public void setProxyKeyUsageMask(int proxyKeyUsageMask) throws IllegalArgumentException
+	{
+		if ((proxyKeyUsageMask & KeyUsage.digitalSignature) == 0)
+			throw new IllegalArgumentException("The digital signature bit must be always set for the proxy");
+		this.proxyKeyUsageMask = proxyKeyUsageMask;
+	}
 
 	/**
 	 * Used to set the type of the proxy. Useful only in case the parent
@@ -178,11 +244,33 @@ public abstract class BaseProxyCertificateOptions
 	/**
 	 * Defines whether the resulting proxy will be a limited proxy. Job
 	 * submission with a limited proxy is not possible.
+	 * <p>
+	 * For legacy proxy this is the only way to control the proxy's application area.
+	 * RFC and draft proxies allows for a more rich and extensible semantics using 
+	 * {@link #setPolicy(ProxyPolicy)}. 
+	 * <p>
+	 * Since version 1.2.0, in case of RFC proxies, usage of this method with argument 'true' is
+	 * equivalent to calling <code>setPolicy(new ProxyPolicy(ProxyPolicy.LIMITED_PROXY_OID))</code>
+	 * and with argument false to <code>setPolicy(new ProxyPolicy(ProxyPolicy.INHERITALL_POLICY_OID))</code>.
+	 * Note that subsequent calls to setPolicy will overwrite the setLimited setting. Therefore the following 
+	 * code:
+	 * <pre>
+	 * param.setLimited(true);
+	 * param.setPolicy(new ProxyPolicy(ProxyPolicy.INHERITALL_POLICY_OID));
+	 * </pre>
+	 * configures the engine to create limited legacy proxies or unlimited rfc proxies. 
+	 * As this behavior is rather not intended it is strongly advised NOT to mix 
+	 * setLimited and setPolicy calls in any case.
+	 * 
 	 * @param limited true if proxy shall be limited
 	 */
 	public void setLimited(boolean limited)
 	{
 		this.limited = limited;
+		if (limited)
+			setPolicy(new ProxyPolicy(ProxyPolicy.LIMITED_PROXY_OID));
+		else
+			setPolicy(new ProxyPolicy(ProxyPolicy.INHERITALL_POLICY_OID));
 	}
 
 	/**
@@ -267,6 +355,8 @@ public abstract class BaseProxyCertificateOptions
 	 * policy. See RFC3820. Policy can be null in case the OID in it self
 	 * defines the behavior, like with "inherit all" policy or
 	 * "independent" policy.
+	 * <p>
+	 * Note: this setting is ignored for legacy proxies.
 	 * @param policy to be set
 	 */
 	public void setPolicy(ProxyPolicy policy)
